use anyhow::Result;
use chrono::SecondsFormat;
use tokio::io::{AsyncWrite, AsyncWriteExt};

use crate::cap::Capabilities;
use crate::handler::{IrcCommand, IrcConnection};
use lavina_core::player::RoomHistoryResult;
use lavina_core::room::RoomId;
use proto_irc::client::ChatHistory;
use proto_irc::server::{ServerMessage, ServerMessageBody};
use proto_irc::{Chan, Recipient, Tag};

impl IrcCommand for ChatHistory {
    async fn handle_with(&self, conn: &mut IrcConnection<'_, impl AsyncWrite + Unpin>) -> Result<()> {
        if !conn.user.enabled_capabilities.contains(Capabilities::ChatHistory) {
            tracing::debug!(
                "Requested chat history for user {:?} even though the capability was not negotiated",
                conn.user.nickname
            );
            return Ok(());
        }
        let channel_name = match self.chan.clone() {
            Chan::Global(chan) => chan,
            // TODO Respond with an error when a local channel is requested
            Chan::Local(chan) => chan,
        };
        let room_id = &RoomId::try_from(channel_name.clone())?;
        let res = conn.player_connection.get_room_message_history(room_id, self.limit).await?;
        match res {
            RoomHistoryResult::Success(messages) => {
                for message in messages {
                    let mut tags = vec![];
                    if conn.user.enabled_capabilities.contains(Capabilities::ServerTime) {
                        let tag = Tag {
                            key: "time".into(),
                            value: Some(message.created_at.to_rfc3339_opts(SecondsFormat::Millis, true).into()),
                        };
                        tags.push(tag);
                    }
                    ServerMessage {
                        tags,
                        sender: Some(message.author_name.into()),
                        body: ServerMessageBody::PrivateMessage {
                            target: Recipient::Chan(self.chan.clone()),
                            body: message.content.into(),
                        },
                    }
                    .write_async(conn.writer)
                    .await?;
                }
                conn.writer.flush().await?;
            }
            RoomHistoryResult::NoSuchRoom => {}
        }
        Ok(())
    }
}
